from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import ManyTranslations as ui
from robot.utils import unic

class ListShouldContainSubListProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['list1', 'list2', 'msg=None', 'values=True'])] = self
    #驗證list2中的所有元件都有被包含在list1中
    def i18n_Proxy(self, func):
        def proxy(self, list1, list2, msg=None, values=True):
            #創出該次呼叫的參數紀錄
            full_args = [str(list1), str(list2)] #將list轉str, 方便之後資料讀寫

            #翻譯
            list1_trans = i18n.I18nListener.MAP.values(list1, full_args)
            list2_trans = i18n.I18nListener.MAP.values(list2, full_args)

            list1_have_multi_trans  = False
            for lt in list1_trans:
                if len(lt) >1:
                    list1_have_multi_trans  = True
                    break
            list2_have_multi_trans  = False
            for lt in list2_trans:
                if len(lt) >1:
                    list2_have_multi_trans  = True
                    break

            #遭遇一詞多譯
            if list1_have_multi_trans or list2_have_multi_trans:
                ListShouldContainSubListProxy.show_warning(self, list1, list2, full_args) #show warning
                #檢查case會pass or fail
                diffs = ', '.join(unic(item) for item in list2 if item not in list1)

                if not diffs: #pass
                    # 對預計開啟的UI做一些準備
                    i18n.I18nListener.Is_Multi_Trans = True
                    
                    for i, lt in enumerate(list1_trans):
                        if len(lt)>1 and str(full_args)+list1[i] not in ui.UI.unique_log: #FIXME dict keys是否要在這邊判斷
                            multi_trans_word = [list1[i]]                            # 還是要移交add_trans_info處理
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            ui.UI.add_trans_info(self, multi_trans_word, lt, full_args, func.__name__)
                    for i, lt in enumerate(list2_trans):
                        if len(lt)>1 and str(full_args)+list2[i] not in ui.UI.unique_log: #FIXME dict keys是否要在這邊判斷
                            multi_trans_word = [list2[i]]                            # 還是要移交add_trans_info處理
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            ui.UI.add_trans_info(self, multi_trans_word, lt, full_args, func.__name__)
            #將處理好的翻譯回傳給robot原生keyword
            return func(self, list1_trans, list2_trans, msg, values)
        return proxy

    def show_warning(self, list1, list2, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_list1 = Proxy().deal_warning_message_for_list(list1, full_args, 'LIST1')
        message_for_list2 = Proxy().deal_warning_message_for_list(list2, full_args,  'LIST2')
        if message_for_list1 or message_for_list2:
            message = language + test_name + message_for_list1 + '\n' + message_for_list2 + '\n'\
                        'You should verify translation is correct!'
            logger.warn(message)


from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import ManyTranslations as ui
from robot.utils import unic

class ListShouldContainValueProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['list_', 'value', 'msg=None'])] = self
        # Fails if the ``value`` is not found from ``list``
    def i18n_Proxy(self, func):
        def proxy(self, list_, value, msg=None):            
            #創出該次呼叫的參數紀錄
            full_args = [str(list_), value] #將list轉str, 方便之後資料讀寫
            
            #翻譯
            list_trans = i18n.I18nListener.MAP.values(list_, full_args)
            value_trans = i18n.I18nListener.MAP.value(value, full_args)
            # logger.warn(list_trans)
            # logger.warn(value_trans)

            list_have_multi_trans = False
            for lt in list_trans:
                if len(lt) >1:
                    list_have_multi_trans  = True
                    break 

            #遭遇一詞多譯
            if list_have_multi_trans or len(value_trans)>1:
                ListShouldContainValueProxy.show_warning(self, list_, value, full_args) #show warning
                #檢查case會pass or fail
                is_pass = False
                if 'not' in func.__name__ :
                    if value not in list_:
                        is_pass = True
                else:
                    if value in list_:
                        is_pass = True
                if is_pass: #pass
                    # 對預計開啟的UI做一些準備
                    i18n.I18nListener.Is_Multi_Trans = True

                    for i, lt in enumerate(list_trans):
                        if len(lt)>1 and str(full_args)+list_[i] not in ui.UI.unique_log: #FIXME dict keys是否要在這邊判斷
                            multi_trans_word = [list_[i]]                            # 還是要移交add_trans_info處理
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            ui.UI.add_trans_info(self, multi_trans_word, lt, full_args, func.__name__)
                    if len(value_trans) > 1 and str(full_args)+value not in ui.UI.unique_log:
                        multiple_translation_word = [value]     
                        ui.UI.origin_xpaths_or_arguments.append(full_args)
                        ui.UI.add_trans_info(self, multiple_translation_word, value_trans, full_args, func.__name__) #將翻譯詞加進等等UI會用到的dictionary中
            #將處理好的翻譯回傳給robot原生keyword
            return func(self, list_trans, value_trans, msg)
        return proxy

    def show_warning(self, list_, value, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_list = Proxy().deal_warning_message_for_list(list_, full_args, 'LIST')
        message_for_value = Proxy().deal_warning_message_for_one_word(value, full_args, 'VALUE')
        if message_for_list or message_for_value:
            message = language + test_name + message_for_list + '\n' + message_for_value + '\n' + 'You should verify translation is correct!'
            logger.warn(message)


from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import ManyTranslations as ui

class ElementTextShouldBeProxy(Proxy): #FIXME 此proxy還有待refactor
    def __init__(self, arg_format):
        arg_format[repr(['locator', 'expected', 'message=None', 'ignore_case=False'])] = self
    
    def i18n_Proxy(self, func):
        def proxy(self, locator, expected, message=None, ignore_case=False): #locator://*[@text='支援'](locator有時並不需要翻譯) expected:support
            
            #創出該呼叫的參數紀錄
            full_args = [locator, expected]

            #翻譯
            possible_translations = i18n.I18nListener.MAP.value(expected, full_args) #support可能map成支援/支援服務

            actual_text = ''
            #遭遇一詞多譯
            if len(possible_translations) > 1:
                ElementTextShouldBeProxy.show_warning(self, expected, full_args)

                #判斷原本case會過還是fail 

                #pass
                i18n.I18nListener.Is_Multi_Trans = True

                if str(full_args)+expected not in ui.UI.unique_log:
                    multiple_translation_words = []
                    multiple_translation_words.append(expected) #將expected word包裝成list格式
                    ui.UI.origin_xpaths_or_arguments.append(full_args)
                    ui.UI.add_trans_info(self, multiple_translation_words, possible_translations, full_args, func.__name__)

                BuiltIn().import_library('SeleniumLibrary')
                actual_text = BuiltIn().run_keyword('Get Text', locator) #有多種翻譯，取得locator的字當作actual text #FIXME 此作法不是很好
            
            else:# 沒有遭遇一詞多譯 或者 有一詞多譯但case會fail
                actual_text = possible_translations[0]
            
            actual_text_message = "'%s'is currently resolved as'%s'\n" %(expected, actual_text)
            logger.info(actual_text_message)
            return func(self, BuiltIn().replace_variables(locator), actual_text, message, ignore_case) if actual_text in possible_translations else func(self, BuiltIn().replace_variables(locator), expected, message, ignore_case)
                                #locator://*[@text='支援'] , expected(support)以修正過後的actual text(支援)代替
        return proxy
    
    def show_warning(self, expected, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_expected = Proxy().deal_warning_message_for_one_word(expected, full_args, 'Expected')
        if message_for_expected :
            if expected not in i18n.I18nListener.Not_SHOW_WARNING_WORDS:   
                message = language + test_name + message_for_expected + ' '*3 + '\n' + 'You should verify translation is correct!'
                logger.warn(message)
            else:
                message = 'Detail Information\n' + language + message_for_expected + ' '*3  + '\nYou had resolved the multiple translations of the word: \'%s\'' %(expected)
                logger.info(message)


from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
from SeleniumLibrary.keywords import AlertKeywords
import I18nListener as i18n
import ManyTranslations as ui

class AlertShouldBePresentProxy(Proxy):  #驗證alert資訊有成功出現
    def __init__(self, arg_format):
        arg_format[repr(['text=\'\'', 'action=\'ACCEPT\'', 'timeout=None'])] = self
                        # text: 腳本輸入的預期文字
                        # Input Text Into Alert 也適用此方法
    def i18n_Proxy(self, func):
        def proxy(self, text='', action='ACCEPT', timeout=None):
            #創出該呼叫的參數紀錄
            full_args = [text] 

            #翻譯
            text_trans = i18n.I18nListener.MAP.value(text, full_args)
            # logger.warn(text_trans)

            #遭遇一詞多譯
            if len(text_trans)>1:
                AlertShouldBePresentProxy.show_warning(self, text, full_args) #show warning

                pass_tt = ''
                #檢查case會pass or fail
                is_pass = False
                if 'present' in func.__name__ :
                    message = AlertKeywords.handle_alert(action, timeout)
                    for tt in text_trans:
                        if tt == message:
                            is_pass = True
                            pass_tt = tt
                            break
                if is_pass: #pass
                    # 對預計開啟的UI做一些準備
                    i18n.I18nListener.Is_Multi_Trans = True

                    if len(text_trans)>1 and str(full_args)+text not in ui.UI.unique_log: #判斷不是重複(arg+翻譯詞都相同)的翻譯
                        multiple_translation_word = [text]     
                        ui.UI.origin_xpaths_or_arguments.append(full_args)
                        ui.UI.add_trans_info(self, multiple_translation_word, text_trans, full_args, func.__name__) #將翻譯詞加進等等UI會用到的dictionary中
                    #有一詞多譯且case通過
                    return func(self, pass_tt, action, timeout)
            # 沒有遭遇一詞多譯 或者 有多個翻譯詞卻case失敗
            # logger.warn(text_trans[0])
            return func(self, text_trans[0], action, timeout)
        return proxy 

    def show_warning(self, text):
        language = 'i18n in %s:\n' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_text = Proxy().deal_warning_message_for_one_word(text, 'Text')
        if message_for_text != '':
            message = language + test_name + message_for_text + ' '*3 + '\n' + 'You should verify translation is correct!'
            logger.warn(message)




from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
from selenium.webdriver.remote.webelement import WebElement
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import sys
import ManyTranslations as ui

class FindElementsProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['by=\'id\'', 'value=None'])] = self
        # 'Page Should Contain Element' 會呼叫此proxy
        # value是要找的element的locator , by(是"xpath:")
    def i18n_Proxy(self, func):
        def proxy(self, by='id', value=None):
            if isinstance(value, WebElement):  #檢查機制
                return func(self, by, value)
            
            #創出該次呼叫的參數紀錄
            full_args = [value]

            #翻譯 ， 針對xpath中需要被翻譯的部分(屬性)
            BuiltIn().import_library('SeleniumLibrary')
            locator = i18n.I18nListener.MAP.locator(BuiltIn().replace_variables(value), full_args) 
                #會呼叫i18nMap的locator(),內部會翻譯xpath內需要翻譯的文字部分，
                #並會設定multiple_translation_words，讓下一行get_multiple_translation_words()取用
            multiple_translation_words = i18n.I18nListener.MAP.get_multiple_translation_words() 

            is_actual = False
            xpath = ''
            #遭遇一詞多譯
            if len(locator) > 1:
                FindElementsProxy.show_warning(self, value, multiple_translation_words, full_args)

                #判斷case會過或fail
                for i, translation_locator in enumerate(locator):
                    #把每種trans_locator用 '|' 串起來， 此種作法之後可以refactor
                    xpath += '|' + translation_locator.replace('xpath:', '') if i != 0 else translation_locator.replace('xpath:', '')
                    #如果畫面上有該翻譯
                    is_actual = BuiltIn().run_keyword_and_return_status('Get WebElement', translation_locator) 
                    if is_actual:
                        break
                    
                if is_actual: #pass
                    # 對預計開啟的UI做一些準備
                    i18n.I18nListener.Is_Multi_Trans = True
                    
                    if str(full_args)+multiple_translation_words[0] not in ui.UI.unique_log:
                        word_translation = i18n.I18nListener.MAP.values(multiple_translation_words, full_args)
                        ui.UI.origin_xpaths_or_arguments.append(full_args)
                        ui.UI.add_trans_info(self, multiple_translation_words, word_translation, full_args, func.__name__)
                        
                    actual_locator_message = "System use the locator:'%s' to run!\n" %translation_locator
                    logger.info(actual_locator_message)
                        
            else:# 沒有遭遇一詞多譯的情況
                xpath = locator[0] #直接選第一個當xpath。否則一詞多譯的xpath是一串由 or 組成的string
            return func(self, by, BuiltIn().replace_variables(xpath))
        return proxy

    def show_warning(self, locator, multiple_translation_words, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = BuiltIn().get_variable_value("${TEST NAME}")
        message_for_words = Proxy().deal_warning_message_for_list(multiple_translation_words, full_args,  'MULTI_TRANS_WORDS')
        message = language + 'Test Name: ' + test_name + '\n' + 'locator: ' + locator +'\n'+ \
              message_for_words + '\n' + 'You should verify translation is correct!'
        if message_for_words:
            logger.warn(message)
            # Screenshot().take_screenshot(width=700)


from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import ManyTranslations as ui

class ListShouldNotContainDuplicatesProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['list_', 'msg=None'])] = self
        # fail if the list has more than one same item.
    def i18n_Proxy(self, func):
        def proxy(self, list_, msg=None):
            #創出該次呼叫的參數紀錄
            full_args = [str(list_)] #將list轉str, 方便之後資料讀寫

            #翻譯
            list_trans = i18n.I18nListener.MAP.values(list_, full_args)

            list_have_multi_trans = False
            for lt in list_trans:
                if len(lt) >1:
                    list_have_multi_trans  = True
                    break 
            
            #遭遇一詞多譯
            if list_have_multi_trans:
                ListShouldNotContainDuplicatesProxy.show_warning(self, list_, full_args)
                #檢查case會pass or fail
                have_dupes = False
                for item in list_:
                    if list_.count(item) >1:
                        have_dupes = True
                
                if not have_dupes: #pass
                    # 對預計開啟的UI做一些準備
                    i18n.I18nListener.Is_Multi_Trans = True

                    for i, lt in enumerate(list_trans):
                        if len(lt)>1 and str(full_args)+list_[i] not in ui.UI.unique_log: #FIXME dict keys是否要在這邊判斷
                            multi_trans_word = [list_[i]]                            # 還是要移交add_trans_info處理
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            ui.UI.add_trans_info(self, multi_trans_word, lt, full_args, func.__name__)
            #將處理好的翻譯回傳給robot原生keyword
            return func(self, list_trans, msg)
        return proxy

    def show_warning(self, list_, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_list = Proxy().deal_warning_message_for_list(list_, full_args ,'LIST')
        if message_for_list:
            message = language + test_name + message_for_list + '\n' + 'You should verify translation is correct!'
            logger.warn(message)